@@ -21,5 +21,5 @@
 
 DATADIR = "${prefix}/share"
 DATADIR = DATADIR.replace(
-    "${prefix}", "/home/mclasen/gnome")
-VERSION = "2.43.4"
+    "${prefix}", "/usr/local")
+VERSION = "2.43.90"
